import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:pc_plugins/controllers/gallery/temp_config.dart';
import 'package:pc_plugins/model/search_data/search_data.dart';

import '../../custom_exception.dart';
import '../../model/algolia_filter_providers.dart';
import '../repository_template.dart';

// final searchingPvd=AutoDisposeStateProviderFamily<bool,String>((_,id)=>false);

class tController<T> extends StateNotifier<AsyncValue<searchData<T>>> {
  final Reader reader;
  final String? uid;
  final repo_tempConfig<T> config;
  String? cachedQuery;
  // String pageid;
  // int defaultPSize;

  tController(this.reader,this.uid,{required this.config}) :super(AsyncValue.data(searchData(data: []))){
    // if(initialLoad)search(null,showNoResult: false);
  }

  // pageInd get searchConfig=>reader(pageIndexPvd(config.pageid));

  Future<void> initialLoad({int? lmt})async{
    if(state.asData==null)await search(null,lmt:lmt,showNoResult: false);
  }

  void setDialog(String s){
    if(config.dialogC!=null) {
      reader(config.dialogC!.state).update((state) => s);}
  }
  void setExDialog(CustomException e){
    if(config.exDialogC!=null) {
      reader(config.exDialogC!.state).update((state) => e);}
  }

  Future<void> loadafter({int? lmt,bool useCQ=false}) async{
    if(state.value?.canNext==true) {
      return search(state.value?.query,page: state.value!.page+1,
          lmt: lmt??config.defaultPSize,useCQ:useCQ);
    } else {
      setDialog("this is the last page :|");}
  }

  Future<void> search(String? s,{int? lmt,int page=0,bool showNoResult=false,bool useCQ=true}) async{
    if(state.whenOrNull(loading: ()=>true)==true||uid==null)return;

    final cached=state;
    if(config.onSearchChange!=null)config.onSearchChange!(reader);
    state=AsyncValue<searchData<T>>.loading();

    final filters=reader(algoFiltersPvd(config.pageid)).values.map((e) => e.function).toList();
    s=useCQ?(s??cachedQuery):s;
    cachedQuery=s;
    try {
      final agdata=config.customSearchFunc!=null?
      await config.customSearchFunc!(s,reader, page: page,hperp: lmt??config.defaultPSize,
              containID: uid,filters: filters):
      await reader(config.dataSrcPvd).searchT_algo(
          s, page: page,hperp: lmt??config.defaultPSize,
          containID: uid,filters: filters);
      if(agdata.pageInfo["nbHits"]==0&&showNoResult){
        setDialog("no result found");
      }

      if(mounted) state = AsyncValue.data(agdata);
    } on CustomException catch (e, st) {
      state=state.copyWithPrevious(cached);
      setExDialog(e);
    }
  }

  Future<void> getSuggest(T c) async{
    if(state.isLoading||uid==null)return;

    if(config.onSearchClick!=null)config.onSearchClick!(reader);
    state=const AsyncValue.loading();
    try {
      // gcs = await reader(fsRepoProvider).getGCs(uid,lmt);
      final data=await reader(config.dataSrcPvd).getbyId(config.getId(c));
      state = AsyncValue.data(searchData.single(data));

    } on CustomException catch (e, st) {
      state=state.copyWithPrevious(state);
      setExDialog(e);
    }
  }

  Future<void> loadb4({int? lmt,bool useCQ=false}) async{
    if(state.value?.canPre==true) {
      return search(state.value?.query,
          page: state.value!.page-1,
          lmt: lmt??config.defaultPSize,useCQ: useCQ);
    } else {
      setDialog("this is the first page :|");}
  }

  updatePvds(T c)async{
    if(!mounted||state.valueOrNull==null)return;

    final i=state.value!.data.indexWhere((e) => config.getId(e)==config.getId(c));
    if(i!=-1){return state=AsyncValue.data(state.value!.copyWith(data: state.value!.data..[i]=c));}

    if(state.value!.data.length<state.value!.hitsPerPage&&
        state.value!.query.isEmpty&&state.value!.getParams(config.scopeFilters??[]).isEmpty){
      state=AsyncValue.data(state.value!.add(c));}
  }

  updateDeletePvds(String id)async{
    if(!mounted||state.valueOrNull==null)return;
    state=state.whenData((v){
      final i=v.data.indexWhere((e) => config.getId(e)==id);
      return v.remove(i);});
  }

  Future<T?> updateT(T c)async{
    try{
      await reader(config.dataSrcPvd).edit(c);
      updatePvds(c);
      return c;
    }on CustomException catch(e,st){
      setExDialog(e);
      return null;
    }
  }

  Future<T?> addT(T c)async{
    try{
      final t=await reader(config.dataSrcPvd).add(c);

      updatePvds(t);
      return t;
    }on CustomException catch(e,st){
      setExDialog(e);
      return null;
    }
  }

  Future<bool> removeT(String id)async{
    try{
      await reader(config.dataSrcPvd).deletebyId(id);
      updateDeletePvds(id);
      return true;
    }on CustomException catch(e,st){
      setExDialog(e);
      return false;
    }
  }

  Future<void> set2read(T c)async{
    try{
      if(uid==null||config.isUnread==null||!config.isUnread!(c,uid!))return;
      final nw=await reader(config.dataSrcPvd).read(c,uid!);
      updatePvds(nw);
    }on CustomException catch(e,st){
      setExDialog(e);
    }
  }
}
