
import 'package:flutter/services.dart';
import 'package:form_builder_validators/form_builder_validators.dart';


const namePattern=r'([a-zA-Z]$)';
const ssn4Pattern=r'(^(?:[+0]9)?[0-9]{4}$)';
const ssn9Pattern=r'(\d{3}-\d{2}-\d{4}$)';
const zipPattern=r'(^(?:[+0]9)?[0-9]{5}$)';
const einPattern=r'(\d{2}-\d{7}$)';
const checkNumberPattern=r'(\d{3,4})';
const accountNumberPattern=r'(\d{4,17})';
const routingNumberPattern=r'(\d{9})';
const onlyNum=r'^[0-9]+$|^$';///or r'^(\s*|\d+)$'
const passwordPattern=r'^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-9])(?=.*?[!@#\$&*~]).{8,}$';
const password12Pattern=r'^.{12,}$';

String? checkPattern(String? input,Pattern pt,{String? errMsg="Please enter valid information"}){
  RegExp regExp = RegExp(pt as String);
  if (input==null||!regExp.hasMatch(input)) {
    return errMsg;}
}

String? pwValidator(String? pw,{bool required=true,
  String invalid="invalid password!",}){
  if(pw==null||pw.isEmpty)return required?invalid:null;

  if(pw.length<8)return invalid;
  return null;
}

String? Function(String?) emailValidator()=>
    FormBuilderValidators.compose([
      FormBuilderValidators.required(),
      FormBuilderValidators.email(errorText: "Please enter valid email")]);

String? checkPassword(String? input){
  if(input==null)return "Please enter Password";
  if (!RegExp(password12Pattern).hasMatch(input)) {
    return "Password must be at least 12 characters";}
  if (!input.contains(RegExp(r'[a-z]'))){
    return "Password must contain a lower case letter";}
  if (!input.contains(RegExp(r'[A-Z]'))){
    return "Password must contain a upper case letter";}
  if (!input.contains(RegExp(r'[0-9]'))){
    return "Password must contain a number";}
  if (!input.contains(RegExp(r'[!@#\$%^&*~(),.?":{}|<>]'))){
    return "Password must contain a special letter";}
  return null;
}

accountNumberFormatter() => LengthLimitingNumberOnlyFormatter(17);
routingNumberFormatter() => LengthLimitingNumberOnlyFormatter(9);

/// Format incoming numeric text to fit the format of XX-XXXXXXX...
class EinFormatter extends TextInputFormatter {
  @override
  TextEditingValue formatEditUpdate(
      TextEditingValue oldValue,
      TextEditingValue newValue
      ) {
    final int newTextLength = newValue.text.length;
    int selectionIndex = newValue.selection.end;
    int usedSubstringIndex = 0;
    final StringBuffer newText = StringBuffer();
    if(newValue.text.contains(RegExp(r'[a-zA-z]'))||newValue.text.length>10){
      return TextEditingValue(text: oldValue.text, selection: TextSelection.collapsed(offset: oldValue.selection.end));}
    if(newValue.text.contains('-')) {
      return TextEditingValue(
        text: newValue.text,
        selection: TextSelection.collapsed(offset: selectionIndex),
      );
    }
    if (newTextLength >= 3) {
      newText.write(newValue.text.substring(0, usedSubstringIndex = 2) + '-');
      if (newValue.selection.end >= 2) { selectionIndex++;}
    }
    if (newTextLength >= usedSubstringIndex) {
      newText.write(newValue.text.substring(usedSubstringIndex));}
    return TextEditingValue(
      text: newText.toString(),
      selection: TextSelection.collapsed(offset: selectionIndex),
    );
  }
}

// number only, length limited
class LengthLimitingNumberOnlyFormatter extends TextInputFormatter {
  final int length;
  LengthLimitingNumberOnlyFormatter(this.length);

  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    if(!RegExp(onlyNum).hasMatch(newValue.text)||newValue.text.length>length) {
      return TextEditingValue(
        text: oldValue.text,
        selection: TextSelection.collapsed(offset: oldValue.selection.end),
      );}
    return TextEditingValue(
      text: newValue.text,
      selection: TextSelection.collapsed(offset: newValue.selection.end),
    );
  }
}

class Ssn9Formatter extends TextInputFormatter {
  @override
  TextEditingValue formatEditUpdate(
      TextEditingValue oldValue,
      TextEditingValue newValue
      ) {
    String ntext=newValue.text.replaceAll("-", "");
    if(ntext.isNotEmpty&&(!RegExp(onlyNum).hasMatch(ntext)||ntext.length>9)) {
      return oldValue;}

    int cursor=newValue.text.substring(0,newValue.selection.end).replaceAll("-", "").length;

    if(ntext.length>3){
      ntext=ntext.substring(0,3)+"-"+ntext.substring(3);
      if(cursor>3)cursor++;
    }
    if(ntext.length>6){
      ntext=ntext.substring(0,6)+"-"+ntext.substring(6);
      if(cursor>6)cursor++;
    }

    return TextEditingValue(
      text: ntext,
      selection: TextSelection.collapsed(offset: cursor),
    );
  }
}
